using System;
using System.Drawing;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using AddinExpress.WD;
using Microsoft.Data.ConnectionUI;
using System.Data.SqlClient;
using Word = Microsoft.Office.Interop.Word;
using System.IO;

namespace WordMerger
{
    public partial class ServerExplorerTaskPane : AddinExpress.WD.ADXWordTaskPane
    {
        DataConnectionDialog dcd;
        string dbName = string.Empty;
        string tableName = string.Empty;
        string serverName = string.Empty;
        string connectionString = string.Empty;

        public ServerExplorerTaskPane()
        {
            InitializeComponent();
            dcd = new DataConnectionDialog();
        }

        private void btnConnect_Click(object sender, EventArgs e)
        {
            DataConnectionConfiguration dcs = new DataConnectionConfiguration(null);
            dcs.LoadConfiguration(dcd);

            if (DataConnectionDialog.Show(dcd) == DialogResult.OK)
            {
                var tables = SqlUtils.GetAllTables(dcd.ConnectionString);

                SqlConnectionStringBuilder builder = new SqlConnectionStringBuilder();
                builder.ConnectionString = dcd.ConnectionString;
                dbName = builder.InitialCatalog;
                serverName = builder.DataSource;
                connectionString = dcd.ConnectionString;

                TreeNode rootNode = new TreeNode(builder.InitialCatalog, 1, 1);
                TreeNode tablesNode = rootNode.Nodes.Add("Tables", "Tables", 2, 2);

                foreach (string table in tables)
                {
                    TreeNode tableNode = tablesNode.Nodes.Add(table, table, 3, 3);
                    tableNode.Tag = "table";
                }

                tvDatabase.Nodes.Add(rootNode);
                tvDatabase.ExpandAll();
            }
        }

        private void tvDatabase_NodeMouseDoubleClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            TreeNode selectedNode = e.Node;
            if (selectedNode.Tag.ToString() == "table")
            {
                var columns = SqlUtils.GetAllColumns(connectionString, selectedNode.Text);
                foreach (string col in columns)
                {
                    TreeNode columnNode = selectedNode.Nodes.Add(col, col, 6, 6);
                    columnNode.Tag = "column";
                }
                tableName = selectedNode.Text.Replace("dbo.", "");
                selectedNode.Expand();
            }
            else if (selectedNode.Tag.ToString() == "column")
            {
                Word.Application wordApp = null;
                Word.Document doc = null;
                Word.Selection selection = null;
                Word.MailMerge wordMerge = null;
                Word.MailMergeFields wordMergeFields = null;

                try
                {
                    wordApp = (Word.Application)WordAppObj;
                    doc = wordApp.ActiveDocument;

                    wordMerge = doc.MailMerge;
                    wordMergeFields = wordMerge.Fields;
                    selection = wordApp.Selection;

                    wordMergeFields.Add(selection.Range, selectedNode.Text);

                }
                finally
                {
                    if (wordMergeFields != null)
                        Marshal.ReleaseComObject(wordMergeFields);
                    if (wordMerge != null)
                        Marshal.ReleaseComObject(wordMerge);
                    if (selection != null)
                        Marshal.ReleaseComObject(selection);
                    if (doc != null)
                        Marshal.ReleaseComObject(doc);
                }
            }
        }

        private void btnDoMerge_Click(object sender, EventArgs e)
        {
            Word.Application wordApp = null;
            Word.Document doc = null;
            Word.MailMerge wordMerge = null;
            Word.MailMergeFields wordMergeFields = null;

            object sqlQuery = string.Empty;
            object connection = string.Empty;
            string odcPath = string.Empty;

            Object oMissing = System.Reflection.Missing.Value;
            Object oFalse = false;

            try
            {
                wordApp = WordAppObj as Word.Application;
                doc = wordApp.ActiveDocument;
                wordMerge = doc.MailMerge;
                wordMergeFields = wordMerge.Fields;

                sqlQuery = String.Format("Select * From \"{0}\"", tableName);
                connection = String.Format(
                    "Provider=SQLOLEDB.1;Integrated Security=SSPI;" +
                    "Persist Security Info=True;Initial Catalog={0};Data Source={1}",
                    dbName, serverName);
                odcPath = CreateODCFile(serverName, dbName);

                wordMerge.OpenDataSource(odcPath, ref oMissing, ref oMissing, ref oMissing,
                  ref oMissing, ref oMissing, ref oMissing, ref oMissing, ref oMissing, ref oMissing,
                  ref oMissing, ref connection, ref sqlQuery, ref oMissing, ref oMissing, ref oMissing);

                wordMerge.Destination = Word.WdMailMergeDestination.wdSendToNewDocument;
                wordMerge.Execute(ref oFalse);
            }
            finally
            {
                if (wordMergeFields != null)
                    Marshal.ReleaseComObject(wordMergeFields);
                if (wordMerge != null)
                    Marshal.ReleaseComObject(wordMerge);
                if (doc != null)
                    Marshal.ReleaseComObject(doc);
            }
        }

        private string CreateODCFile(string serverName, string databaseName)
        {
            string odcContent = WordMerger.Properties.Resources.ODCTemplate;
            odcContent = odcContent.Replace("@ServerName", serverName);
            odcContent = odcContent.Replace("@DatabaseName", databaseName);
            string path = Path.Combine(Path.GetTempPath(), String.Format(@"{0}.odc", databaseName));
            File.WriteAllText(path, odcContent);
            return path;
        }
    }
}
